@@ -17,29 +17,20 @@ limitations under the License.
 package common
 
 import (
+	"context"
 	"fmt"
 	"time"
 
 	"github.com/tektoncd/operator/pkg/apis/operator/v1alpha1"
 	informer "github.com/tektoncd/operator/pkg/client/informers/externalversions/operator/v1alpha1"
-	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 )
 
 var (
+	Interval = 10 * time.Second
+	Timeout  = 1 * time.Minute
 	// DefaultSA is the default service account
-	DefaultSA             = "pipeline"
-	PipelineResourceName  = "pipeline"
-	TriggerResourceName   = "trigger"
-	DashboardResourceName = "dashboard"
-	AddonResourceName     = "addon"
-	ConfigResourceName    = "config"
-	ResultResourceName    = "result"
-	ProfileLite           = "lite"
-	ProfileBasic          = "basic"
-	ProfileAll            = "all"
-	Interval              = 10 * time.Second
-	Timeout               = 1 * time.Minute
+	DefaultSA = "pipeline"
 )
 
 const (
@@ -58,17 +49,32 @@ func PipelineReady(informer informer.TektonPipelineInformer) (*v1alpha1.TektonPi
 		}
 		return nil, err
 	}
-
-	if len(ppln.Status.Conditions) != 0 {
-		if ppln.Status.Conditions[0].Status != corev1.ConditionTrue {
-			return nil, fmt.Errorf(PipelineNotReady)
-		}
+	upgradePending, err := CheckUpgradePending(ppln)
+	if err != nil {
+		return nil, err
+	}
+	if upgradePending {
+		return nil, v1alpha1.DEPENDENCY_UPGRADE_PENDING_ERR
+	}
+	if !ppln.Status.IsReady() {
+		return nil, fmt.Errorf(PipelineNotReady)
 	}
 	return ppln, nil
 }
 
+func PipelineTargetNamspace(informer informer.TektonPipelineInformer) (string, error) {
+	ppln, err := getPipelineRes(informer)
+	if err != nil {
+		if apierrors.IsNotFound(err) {
+			return "", nil
+		}
+		return "", err
+	}
+	return ppln.Spec.TargetNamespace, nil
+}
+
 func getPipelineRes(informer informer.TektonPipelineInformer) (*v1alpha1.TektonPipeline, error) {
-	res, err := informer.Lister().Get(PipelineResourceName)
+	res, err := informer.Lister().Get(v1alpha1.PipelineResourceName)
 	return res, err
 }
 
@@ -80,16 +86,36 @@ func TriggerReady(informer informer.TektonTriggerInformer) (*v1alpha1.TektonTrig
 		}
 		return nil, err
 	}
-
-	if len(trigger.Status.Conditions) != 0 {
-		if trigger.Status.Conditions[0].Status != corev1.ConditionTrue {
-			return nil, fmt.Errorf(TriggerNotReady)
-		}
+	upgradePending, err := CheckUpgradePending(trigger)
+	if err != nil {
+		return nil, err
+	}
+	if upgradePending {
+		return nil, v1alpha1.DEPENDENCY_UPGRADE_PENDING_ERR
+	}
+	if !trigger.Status.IsReady() {
+		return nil, fmt.Errorf(TriggerNotReady)
 	}
 	return trigger, nil
 }
 
 func getTriggerRes(informer informer.TektonTriggerInformer) (*v1alpha1.TektonTrigger, error) {
-	res, err := informer.Lister().Get(TriggerResourceName)
+	res, err := informer.Lister().Get(v1alpha1.TriggerResourceName)
 	return res, err
 }
+
+func CheckUpgradePending(tc v1alpha1.TektonComponent) (bool, error) {
+	labels := tc.GetLabels()
+	ver, ok := labels[v1alpha1.ReleaseVersionKey]
+	if !ok {
+		return true, nil
+	}
+	operatorVersion, err := OperatorVersion(context.TODO())
+	if err != nil {
+		return false, err
+	}
+	if ver != operatorVersion {
+		return true, nil
+	}
+	return false, nil
+}
