@@ -17,36 +17,45 @@ limitations under the License.
 package tektoninstallerset
 
 import (
+	"context"
 	"fmt"
 	"strings"
 
 	mf "github.com/manifestival/manifestival"
-	"github.com/tektoncd/operator/pkg/reconciler/common"
+	"github.com/tektoncd/operator/pkg/apis/operator/v1alpha1"
+	"github.com/tektoncd/operator/pkg/reconciler/shared/hash"
 	appsv1 "k8s.io/api/apps/v1"
+	batchv1 "k8s.io/api/batch/v1"
 	corev1 "k8s.io/api/core/v1"
 	apierrs "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/client-go/kubernetes/scheme"
+	"knative.dev/pkg/logging"
 	"knative.dev/pkg/ptr"
 )
 
 const (
-	replicasForHash    = 999
-	lastAppliedHashKey = "operator.tekton.dev/last-applied-hash"
+	replicasForHash = 999
 )
 
 var (
 	namespacePred                      = mf.ByKind("Namespace")
 	configMapPred                      = mf.ByKind("ConfigMap")
+	pvcPred                            = mf.ByKind("PersistentVolumeClaim")
+	jobPred                            = mf.ByKind("Job")
 	secretPred                         = mf.ByKind("Secret")
 	deploymentPred                     = mf.ByKind("Deployment")
 	servicePred                        = mf.ByKind("Service")
 	serviceAccountPred                 = mf.ByKind("ServiceAccount")
+	cronJobPred                        = mf.ByKind("CronJob")
+	eventListenerPred                  = mf.ByKind("EventListener")
+	triggerBindingPred                 = mf.ByKind("TriggerBinding")
+	triggerTemplatePred                = mf.ByKind("TriggerTemplate")
 	rolePred                           = mf.ByKind("Role")
 	roleBindingPred                    = mf.ByKind("RoleBinding")
 	clusterRolePred                    = mf.ByKind("ClusterRole")
 	clusterRoleBindingPred             = mf.ByKind("ClusterRoleBinding")
-	podSecurityPolicyPred              = mf.ByKind("PodSecurityPolicy")
 	validatingWebhookConfigurationPred = mf.ByKind("ValidatingWebhookConfiguration")
 	mutatingWebhookConfigurationPred   = mf.ByKind("MutatingWebhookConfiguration")
 	horizontalPodAutoscalerPred        = mf.ByKind("HorizontalPodAutoscaler")
@@ -60,6 +69,7 @@ var (
 	serviceMonitorPred         = mf.ByKind("ServiceMonitor")
 	routePred                  = mf.ByKind("Route")
 	consoleCLIDownloadPred     = mf.ByKind("ConsoleCLIDownload")
+	consoleLinkHubPred         = mf.ByKind("ConsoleLink")
 	consoleQuickStartPred      = mf.ByKind("ConsoleQuickStart")
 	ConsoleYAMLSamplePred      = mf.ByKind("ConsoleYAMLSample")
 )
@@ -68,36 +78,63 @@ type installer struct {
 	Manifest mf.Manifest
 }
 
-func (i *installer) EnsureCRDs() error {
-	if err := i.Manifest.Filter(mf.Any(mf.CRDs)).Apply(); err != nil {
+func ensureResources(mani *mf.Manifest) error {
+	freshCreate := true
+	// check if all resources in a given set of resources exists
+	ok, err := allResourcesExists(mani)
+	if err != nil {
+		if !apierrs.IsNotFound(err) {
+			return err
+		}
+	}
+	// if error == NotFound error or !ok
+	// then Apply all resources
+	// if ok, that means the reosource already exists
+	// but this reconcile could be an modification eg: concig-defaults change
+	// set freshCreate flag to false, and then Apply all
+	// so that we can skip (break) RECONCILE_AGAIN loop path
+	if ok {
+		freshCreate = false
+	}
+
+	if err := mani.Apply(); err != nil {
 		return err
 	}
+	// on err == nil after Apply() return RECONCILE_AGAIN
+	// if freshCreate == true return RECONCILE_AGAIN
+	// this ensures freshly created resources are in place before proceeding to next stages of reconciler
+	if freshCreate {
+		return v1alpha1.RECONCILE_AGAIN_ERR
+	}
 	return nil
 }
 
+func (i *installer) EnsureCRDs() error {
+	resourceList := i.Manifest.Filter(mf.Any(mf.CRDs))
+	return ensureResources(&resourceList)
+}
+
 func (i *installer) EnsureClusterScopedResources() error {
-	if err := i.Manifest.Filter(
+	resourceList := i.Manifest.Filter(
 		mf.Any(
 			namespacePred,
 			clusterRolePred,
-			podSecurityPolicyPred,
 			validatingWebhookConfigurationPred,
 			mutatingWebhookConfigurationPred,
 			clusterInterceptorPred,
 			clusterTaskPred,
 			clusterTriggerBindingPred,
 			consoleCLIDownloadPred,
+			consoleLinkHubPred,
 			consoleQuickStartPred,
 			ConsoleYAMLSamplePred,
 			securityContextConstraints,
-		)).Apply(); err != nil {
-		return err
-	}
-	return nil
+		))
+	return ensureResources(&resourceList)
 }
 
 func (i *installer) EnsureNamespaceScopedResources() error {
-	if err := i.Manifest.Filter(
+	resourceList := i.Manifest.Filter(
 		mf.Any(
 			serviceAccountPred,
 			clusterRoleBindingPred,
@@ -105,13 +142,19 @@ func (i *installer) EnsureNamespaceScopedResources() error {
 			roleBindingPred,
 			configMapPred,
 			secretPred,
+			pvcPred,
+			jobPred,
 			horizontalPodAutoscalerPred,
 			pipelinePred,
 			serviceMonitorPred,
-		)).Apply(); err != nil {
-		return err
-	}
-	return nil
+			cronJobPred,
+			eventListenerPred,
+			triggerBindingPred,
+			triggerTemplatePred,
+			servicePred,
+			routePred,
+		))
+	return ensureResources(&resourceList)
 }
 
 func (i *installer) EnsureDeploymentResources() error {
@@ -122,13 +165,6 @@ func (i *installer) EnsureDeploymentResources() error {
 		}
 	}
 
-	if err := i.Manifest.Filter(
-		mf.Any(
-			servicePred,
-			routePred,
-		)).Apply(); err != nil {
-		return err
-	}
 	return nil
 }
 
@@ -140,7 +176,7 @@ func computeDeploymentHash(d appsv1.Deployment) (string, error) {
 	// done to the deployment spec
 	d.Spec.Replicas = ptr.Int32(replicasForHash)
 
-	return common.ComputeHashOf(d.Spec)
+	return hash.Compute(d.Spec)
 }
 
 func (i *installer) createDeployment(expected *unstructured.Unstructured) error {
@@ -159,7 +195,7 @@ func (i *installer) createDeployment(expected *unstructured.Unstructured) error
 	if len(dep.Annotations) == 0 {
 		dep.Annotations = map[string]string{}
 	}
-	dep.Annotations[lastAppliedHashKey] = hash
+	dep.Annotations[v1alpha1.LastAppliedHashKey] = hash
 
 	unstrObj, err := runtime.DefaultUnstructuredConverter.ToUnstructured(dep)
 	if err != nil {
@@ -188,7 +224,7 @@ func (i *installer) updateDeployment(existing *unstructured.Unstructured, existi
 		existingDeployment.Annotations = map[string]string{}
 	}
 
-	existingDeployment.Annotations[lastAppliedHashKey] = newHash
+	existingDeployment.Annotations[v1alpha1.LastAppliedHashKey] = newHash
 
 	unstrObj, err := runtime.DefaultUnstructuredConverter.ToUnstructured(existingDeployment)
 	if err != nil {
@@ -196,7 +232,11 @@ func (i *installer) updateDeployment(existing *unstructured.Unstructured, existi
 	}
 	existing.SetUnstructuredContent(unstrObj)
 
-	return i.Manifest.Client.Update(existing)
+	err = i.Manifest.Client.Update(existing)
+	if err != nil {
+		return v1alpha1.RECONCILE_AGAIN_ERR
+	}
+	return err
 }
 
 func (i *installer) ensureDeployment(expected *unstructured.Unstructured) error {
@@ -207,7 +247,11 @@ func (i *installer) ensureDeployment(expected *unstructured.Unstructured) error
 
 		// If deployment doesn't exist, then create new
 		if apierrs.IsNotFound(err) {
-			return i.createDeployment(expected)
+			errInner := i.createDeployment(expected)
+			if errInner == nil {
+				return v1alpha1.RECONCILE_AGAIN_ERR
+			}
+			return errInner
 		}
 		return err
 	}
@@ -231,7 +275,7 @@ func (i *installer) ensureDeployment(expected *unstructured.Unstructured) error
 		return fmt.Errorf("failed to compute hash of existing deployment: %v", err)
 	}
 
-	hashFromAnnotation, hashExist := existingDeployment.Annotations[lastAppliedHashKey]
+	hashFromAnnotation, hashExist := existingDeployment.Annotations[v1alpha1.LastAppliedHashKey]
 
 	// if hash doesn't exist then update the deployment with hash
 	if !hashExist {
@@ -314,6 +358,27 @@ func (i *installer) AllDeploymentsReady() error {
 	return nil
 }
 
+func (i *installer) IsJobCompleted(ctx context.Context, labels map[string]string, installSetName string) error {
+	for _, u := range i.Manifest.Filter(jobPred).Resources() {
+		resource, err := i.Manifest.Client.Get(&u)
+		if err != nil {
+			return err
+		}
+		job := &batchv1.Job{}
+		if err := scheme.Scheme.Convert(resource, job, nil); err != nil {
+			return err
+		}
+
+		logger := logging.FromContext(ctx)
+		if !isJobCompleted(job) {
+			logger.Info("job not ready in installerset, name: %s, created-by: %s, in namespace: %s", installSetName, labels[v1alpha1.CreatedByKey], job.GetNamespace())
+			return fmt.Errorf("Job not successful")
+		}
+	}
+
+	return nil
+}
+
 func (i *installer) isDeploymentReady(d *unstructured.Unstructured) error {
 
 	resource, err := i.Manifest.Client.Get(d)
@@ -342,3 +407,34 @@ func isDeploymentAvailable(d *appsv1.Deployment) bool {
 	}
 	return false
 }
+
+func allResourcesExists(m *mf.Manifest) (bool, error) {
+	c := m.Client
+	for _, item := range m.Resources() {
+		ok, err := resourceExists(c, &item)
+		if err != nil {
+			return false, err
+		}
+		if !ok {
+			return false, nil
+		}
+	}
+	return true, nil
+}
+
+func resourceExists(c mf.Client, u *unstructured.Unstructured) (bool, error) {
+	_, err := c.Get(u)
+	if err != nil {
+		return false, err
+	}
+	return true, nil
+}
+
+func isJobCompleted(d *batchv1.Job) bool {
+	for _, c := range d.Status.Conditions {
+		if c.Type == batchv1.JobComplete && c.Status == corev1.ConditionTrue {
+			return true
+		}
+	}
+	return false
+}
